#Model Processor
function FetchDataToInput(md::model,inputs::Inputs,elements::Vector{Tria},data::Vector{Float64},enum::IssmEnum) #{{{
	for i in 1:length(elements)
		InputCreate(elements[i],inputs,data,enum)
	end
end#}}}
function ModelProcessor(md::model) #{{{

	#Initialize structures
	elements    = Vector{Tria}(undef,0)
	vertices    = Vector{Vertex}(undef,0)
	nodes       = Vector{Node}(undef,0)
	constraints = Vector{Constraint}(undef,0)
	parameters  = Parameters(Dict{IssmEnum,Parameter}())
	inputs      = Inputs(md.mesh.numberofelements,md.mesh.numberofvertices,Dict{IssmEnum,Input}())

	#Create  elements, vertices and materials (independent of the analysis)
	CreateElements(elements,md)
	CreateVertices(vertices,md)
	CreateParameters(parameters,md)
	CreateInputs(inputs,elements,md)

	#Now create analysis specific data structure
	analysis = StressbalanceAnalysis()
	UpdateParameters(analysis,parameters,md)
	CreateNodes(analysis,nodes,md)
	UpdateElements(analysis,elements,inputs,md)
	CreateConstraints(analysis,constraints,md)

	#Configure objects
	ConfigureObjectx(elements,nodes,vertices,parameters,inputs)

	#Constrain and Number nodes
	SpcNodesx(nodes,constraints,parameters)
	NodesDofx(nodes,parameters)
	error("STOP")

	#Build FemModel
	return FemModel(elements,vertices,nodes,parameters,inputs,constraints)
end# }}}
function CreateElements(elements::Vector{Tria},md::model) #{{{

	#Make sure elements is currently empty
	@assert length(elements)==0

	tempparams   = Parameters(Dict{IssmEnum,Parameter}())
	tempinputs   = Inputs(-1,-1,Dict{IssmEnum,Input}())
	tempvertices = Vector{Vertex}(undef,3)
	tempnodes    = Vector{Node}(undef,3)

	count = 0
	for i in 1:md.mesh.numberofelements

		#Assume Linear Elements for now
		vertexids = md.mesh.elements[i,:]
		nodeids   = md.mesh.elements[i,:]

		#Call constructor and add to dataset elements
		push!(elements,Tria(i,count,vertexids,tempvertices,nodeids,tempnodes,tempparams,tempinputs))
	end

end# }}}
function CreateVertices(vertices::Vector{Vertex},md::model) #{{{

	#Make sure vertices is currently empty
	@assert length(vertices)==0

	#Get data from md
	x = md.mesh.x
	y = md.mesh.y

	count = 0
	for i in 1:md.mesh.numberofvertices
		push!(vertices,Vertex(i,x[i],y[i]))
	end

end# }}}
function CreateParameters(parameters::Parameters,md::model) #{{{

	#Get data from md
	AddParam(parameters,md.materials.rho_ice,MaterialsRhoIceEnum)
	AddParam(parameters,md.materials.rho_water,MaterialsRhoSeawaterEnum)
	AddParam(parameters,md.materials.rho_freshwater,MaterialsRhoFreshwaterEnum)
end# }}}
function CreateInputs(inputs::Inputs,elements::Vector{Tria},md::model) #{{{

	#Only assume we have Matice for now
	FetchDataToInput(md,inputs,elements,md.materials.rheology_B,MaterialsRheologyBEnum)
	FetchDataToInput(md,inputs,elements,md.materials.rheology_n,MaterialsRheologyNEnum)
end# }}}

#Other modules
function ConfigureObjectx(elements::Vector{Tria},nodes::Vector{Node},vertices::Vector{Vertex},parameters::Parameters,inputs::Inputs) #{{{

	for i in 1:length(elements)
		Configure(elements[i],nodes,vertices,parameters,inputs)
	end

end# }}}
function SpcNodesx(nodes::Vector{Node},constraints::Vector{Constraint},parameters::Parameters) #{{{

	for i in 1:length(constraints)
		ConstrainNode(constraints[i],nodes,parameters)
	end

end# }}}
function NodesDofx(nodes::Vector{Node}, parameters::Parameters) #{{{

	error("not implemented yet")

end# }}}
